@@ -27,7 +27,7 @@
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
  */
-/* $PostgreSQL: pgsql/contrib/pgcrypto/imath.c,v 1.9 2010/04/02 15:21:20 mha Exp $ */
+/* contrib/pgcrypto/imath.c */
 
 #include "postgres.h"
 #include "px.h"
@@ -818,8 +818,7 @@ mp_int_mul(mp_int a, mp_int b, mp_int c)
 	 */
 	ua = MP_USED(a);
 	ub = MP_USED(b);
-	osize = MAX(ua, ub);
-	osize = 4 * ((osize + 1) / 2);
+	osize = ua + ub;
 
 	if (c == a || c == b)
 	{
@@ -908,7 +907,7 @@ mp_int_sqr(mp_int a, mp_int c)
 	CHECK(a != NULL && c != NULL);
 
 	/* Get a temporary buffer big enough to hold the result */
-	osize = (mp_size) 4 *((MP_USED(a) + 1) / 2);
+	osize = (mp_size) 2 *MP_USED(a);
 
 	if (a == c)
 	{
@@ -2606,8 +2605,8 @@ s_kmul(mp_digit *da, mp_digit *db, mp_digit *dc,
 		 * Now we'll get t1 = a0b0 and t2 = a1b1, and subtract them out so
 		 * that we're left with only the pieces we want:  t3 = a1b0 + a0b1
 		 */
-		ZERO(t1, buf_size);
-		ZERO(t2, buf_size);
+		ZERO(t1, bot_size + 1);
+		ZERO(t2, bot_size + 1);
 		(void) s_kmul(da, db, t1, bot_size, bot_size);	/* t1 = a0 * b0 */
 		(void) s_kmul(a_top, b_top, t2, at_size, bt_size);		/* t2 = a1 * b1 */
 
@@ -2617,13 +2616,11 @@ s_kmul(mp_digit *da, mp_digit *db, mp_digit *dc,
 
 		/* Assemble the output value */
 		COPY(t1, dc, buf_size);
-		carry = s_uadd(t3, dc + bot_size, dc + bot_size,
-					   buf_size + 1, buf_size);
-		assert(carry == 0);
+		(void) s_uadd(t3, dc + bot_size, dc + bot_size,
+					  buf_size + 1, buf_size + 1);
 
-		carry = s_uadd(t2, dc + 2 * bot_size, dc + 2 * bot_size,
-					   buf_size, buf_size);
-		assert(carry == 0);
+		(void) s_uadd(t2, dc + 2 * bot_size, dc + 2 * bot_size,
+					  buf_size, buf_size);
 
 		s_free(t1);				/* note t2 and t3 are just internal pointers
 								 * to t1 */
@@ -3310,10 +3307,7 @@ s_embar(mp_int a, mp_int b, mp_int m, mp_int mu, mp_int c)
 	dbt = db + MP_USED(b) - 1;
 
 	while (last < 3)
-	{
-		SETUP(mp_int_init_size(TEMP(last), 4 * umu), last);
-		ZERO(MP_DIGITS(TEMP(last - 1)), MP_ALLOC(TEMP(last - 1)));
-	}
+		SETUP(mp_int_init_size(TEMP(last), 2 * umu), last);
 
 	(void) mp_int_set_value(c, 1);
 
