@@ -26,7 +26,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $PostgreSQL: pgsql/contrib/pgcrypto/px.c,v 1.18 2009/06/11 14:48:52 momjian Exp $
+ * contrib/pgcrypto/px.c
  */
 
 #include "postgres.h"
@@ -168,14 +168,12 @@ combo_init(PX_Combo *cx, const uint8 *key, unsigned klen,
 		   const uint8 *iv, unsigned ivlen)
 {
 	int			err;
-	unsigned	bs,
-				ks,
+	unsigned	ks,
 				ivs;
 	PX_Cipher  *c = cx->cipher;
 	uint8	   *ivbuf = NULL;
 	uint8	   *keybuf;
 
-	bs = px_cipher_block_size(c);
 	ks = px_cipher_key_size(c);
 
 	ivs = px_cipher_iv_size(c);
@@ -211,7 +209,6 @@ combo_encrypt(PX_Combo *cx, const uint8 *data, unsigned dlen,
 	int			err = 0;
 	uint8	   *bbuf;
 	unsigned	bs,
-				maxlen,
 				bpos,
 				i,
 				pad;
@@ -219,7 +216,6 @@ combo_encrypt(PX_Combo *cx, const uint8 *data, unsigned dlen,
 	PX_Cipher  *c = cx->cipher;
 
 	bbuf = NULL;
-	maxlen = *rlen;
 	bs = px_cipher_block_size(c);
 
 	/* encrypt */
