@@ -3,12 +3,12 @@
  * restrictinfo.c
  *	  RestrictInfo node manipulation routines.
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/optimizer/util/restrictinfo.c,v 1.60.2.1 2009/07/06 18:26:39 tgl Exp $
+ *	  src/backend/optimizer/util/restrictinfo.c
  *
  *-------------------------------------------------------------------------
  */
@@ -152,7 +152,7 @@ make_restrictinfo_from_bitmapqual(Path *bitmapqual,
 		/*
 		 * Here, we only detect qual-free subplans.  A qual-free subplan would
 		 * cause us to generate "... OR true ..."  which we may as well reduce
-		 * to just "true".	We do not try to eliminate redundant subclauses
+		 * to just "true".  We do not try to eliminate redundant subclauses
 		 * because (a) it's not as likely as in the AND case, and (b) we might
 		 * well be working with hundreds or even thousands of OR conditions,
 		 * perhaps from a long IN list.  The performance of list_append_unique
@@ -249,7 +249,7 @@ make_restrictinfo_from_bitmapqual(Path *bitmapqual,
 				 * We know that the index predicate must have been implied by
 				 * the query condition as a whole, but it may or may not be
 				 * implied by the conditions that got pushed into the
-				 * bitmapqual.	Avoid generating redundant conditions.
+				 * bitmapqual.  Avoid generating redundant conditions.
 				 */
 				if (!predicate_implied_by(list_make1(pred), result))
 					result = lappend(result,
@@ -292,15 +292,15 @@ make_restrictinfos_from_actual_clauses(PlannerInfo *root,
 
 	foreach(l, clause_list)
 	{
-		Expr   *clause = (Expr *) lfirst(l);
-		bool	pseudoconstant;
+		Expr	   *clause = (Expr *) lfirst(l);
+		bool		pseudoconstant;
 		RestrictInfo *rinfo;
 
 		/*
 		 * It's pseudoconstant if it contains no Vars and no volatile
 		 * functions.  We probably can't see any sublinks here, so
-		 * contain_var_clause() would likely be enough, but for safety
-		 * use contain_vars_of_level() instead.
+		 * contain_var_clause() would likely be enough, but for safety use
+		 * contain_vars_of_level() instead.
 		 */
 		pseudoconstant =
 			!contain_vars_of_level((Node *) clause, 0) &&
@@ -391,7 +391,7 @@ make_restrictinfo_internal(Expr *clause,
 
 	/*
 	 * Fill in all the cacheable fields with "not yet set" markers. None of
-	 * these will be computed until/unless needed.	Note in particular that we
+	 * these will be computed until/unless needed.  Note in particular that we
 	 * don't mark a binary opclause as mergejoinable or hashjoinable here;
 	 * that happens only if it appears in the right context (top level of a
 	 * joinclause list).
@@ -669,7 +669,7 @@ select_nonredundant_join_clauses(PlannerInfo *root,
 		 * OK because we're only trying to prove we can dispense with some
 		 * join quals; failing to prove that doesn't result in an incorrect
 		 * plan.  It's quite unlikely that a join qual could be proven
-		 * redundant by an index predicate anyway.	(Also, if we did manage to
+		 * redundant by an index predicate anyway.  (Also, if we did manage to
 		 * prove it, we'd have to have a special case for update targets; see
 		 * notes about EvalPlanQual testing in create_indexscan_plan().)
 		 */
