@@ -3,12 +3,12 @@
  * bufpage.c
  *	  POSTGRES standard buffer page code.
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/storage/page/bufpage.c,v 1.83 2010/01/02 16:57:52 momjian Exp $
+ *	  src/backend/storage/page/bufpage.c
  *
  *-------------------------------------------------------------------------
  */
