@@ -1,9 +1,9 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2010, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2011, PostgreSQL Global Development Group
  *
- * $PostgreSQL: pgsql/src/bin/psql/print.c,v 1.128 2010/07/06 19:19:00 momjian Exp $
+ * src/bin/psql/print.c
  */
 #include "postgres_fe.h"
 
@@ -808,7 +808,7 @@ print_aligned_text(const printTableContent *cont, FILE *fout)
 					unsigned int nbspace;
 
 					if (opt_border != 0 ||
-						(format->wrap_right_border == false && i > 0))
+						(!format->wrap_right_border && i > 0))
 						fputs(curr_nl_line ? format->header_nl_left : " ",
 							  fout);
 
@@ -829,7 +829,7 @@ print_aligned_text(const printTableContent *cont, FILE *fout)
 					else
 						fprintf(fout, "%*s", width_wrap[i], "");
 
-					if (opt_border != 0 || format->wrap_right_border == true)
+					if (opt_border != 0 || format->wrap_right_border)
 						fputs(!header_done[i] ? format->header_nl_right : " ",
 							  fout);
 
